library ieee;
use ieee.std_logic_1164.all;

library work;
use work.jddr_pkg.all;

entity jddr_interface IS
  generic ( EXTRA_WRITE_DELAY : natural );
  port (
    -- Globals Signals
    clk     : IN std_logic;
    reset_n : IN std_logic;

    -- DDR local side interface signals
    transrequest         : IN std_logic;
    readwriten           : IN std_logic;
    ddr_address          : IN STD_LOGIC_VECTOR(16 downto 0);
    transcomplete	 : OUT STD_LOGIC;
    ddr_start_block      : IN STD_LOGIC_VECTOR(6 downto 0);
    ddr_end_block        : IN STD_LOGIC_VECTOR(6 downto 0);

    -- Local side data interface
    read_data    : OUT STD_LOGIC_VECTOR(143 downto 0);
    write_data   : IN STD_LOGIC_VECTOR(143 downto 0);
    data_available : OUT STD_LOGIC;
    data_wanted    : OUT STD_LOGIC;
    data_clk     : OUT STD_LOGIC;

    -- DDR Module Interface Pin Signals
    tm4_ddr_DQ     : inout std_logic_vector(71 downto 0);
    tm4_ddr_DQS    : inout std_logic_vector(8 downto 0);
    tm4_ddr_DM     : out   std_logic_vector(8 downto 0);
    tm4_ddr_clk    : out   std_logic_vector(2 downto 0);
    tm4_ddr_A      : out std_logic_vector(13 downto 0);
    tm4_ddr_RASn   : out std_logic;
    tm4_ddr_CASn   : out std_logic;
    tm4_ddr_WEn    : out std_logic;
    tm4_ddr_Sn     : out std_logic_vector(1 downto 0);
    tm4_ddr_BA     : out std_logic_vector(1 downto 0);
    tm4_ddr_CLKE   : out std_logic_vector(1 downto 0);
    tm4_ddr_RESETn : out std_logic;
    tm4_dqsclk_in   : in std_logic;
    tm4_dqsclk_out   : out std_logic
  );
END jddr_interface;

ARCHITECTURE rtl OF jddr_interface IS
  -- PLL generated clock signals
  signal dqs_delay_ctrl : std_logic;
  signal dqs_outclk    : std_logic;
  signal write_clk     : std_logic;
  signal resynch_clk   : std_logic;
  signal postamble_clk : std_logic;

  -- DDR Datapath interface signals
  SIGNAL wdata_low, wdata_high : STD_LOGIC_VECTOR(71 downto 0);
  SIGNAL rdata_low, rdata_high : STD_LOGIC_VECTOR(71 downto 0);
  SIGNAL rdata_low_r, rdata_high_r : STD_LOGIC_VECTOR(71 downto 0);
  SIGNAL doing_wr, doing_rd : STD_LOGIC;

BEGIN
  tm4_ddr_RESETn <= '1';

  data_clk <= write_clk;
  wdata_low <= write_data(71 downto 0);
  wdata_high <= write_data(143 downto 72);

  PROCESS (write_clk)
  BEGIN
    IF falling_edge(write_clk) THEN
      rdata_low_r <= rdata_low;
      rdata_high_r <= rdata_high;
    END IF;
    IF rising_edge(write_clk) THEN
      read_data(71 downto 0) <= rdata_high_r;
      read_data(143 downto 72) <= rdata_low_r;
    END IF;
  END PROCESS;

  ddrcont : jddr_controller
    generic map ( EXTRA_WRITE_DELAY => EXTRA_WRITE_DELAY )
    port map ( data_available, data_wanted, doing_wr, doing_rd,
               tm4_ddr_ClkE, tm4_ddr_Sn, tm4_ddr_RASn, tm4_ddr_CASn,
               tm4_ddr_WEn, tm4_ddr_A, tm4_ddr_BA,
               transrequest, readwriten, transcomplete,
               ddr_address, ddr_start_block, ddr_end_block, 
               reset_n, write_clk);

  ddr_pll : jddr_pll
    port map( 
      inclk0 => clk, 
      c0 => write_clk,
      c1 => dqs_outclk,
      c2 => postamble_clk,
      c3 => resynch_clk,
      e0 => tm4_ddr_clk(0),
      e1 => tm4_dqsclk_out,
      e2 => tm4_ddr_clk(1),
      e3 => tm4_ddr_clk(2)
    );

 g_datapath : jddr_datapath
    port map (
      clk            => write_clk,
      reset_n        => reset_n,
      dqs_outclk     => dqs_outclk,
      write_clk      => write_clk,
      resynch_clk    => resynch_clk,
      postamble_clk  => postamble_clk,
      dqs_delay_ctrl => dqs_delay_ctrl,

      doing_wr    => doing_wr,
      wdata_low   => wdata_low,
      wdata_high  => wdata_high,
      doing_rd    => doing_rd,
      rdata_low   => rdata_low,
      rdata_high  => rdata_high,
      ddr_dq  => tm4_ddr_DQ,
      ddr_dqs => tm4_ddr_DQS,
      ddr_dm  => tm4_ddr_DM
    );

--------------------------------------------------------------
--  Instantiate Stratix DLL for Read DQS Phase shift
--------------------------------------------------------------

  lpm_ddrcore_strx_dll1 :  stratix_dll
    GENERIC MAP (
      INPUT_FREQUENCY => "7500ps",
      PHASE_SHIFT => "72",
      SIM_INVALID_LOCK => 10000,
      SIM_VALID_LOCK => 1
    )
    PORT MAP (
      clk => tm4_dqsclk_in,
      delayctrlout => dqs_delay_ctrl
    );

END rtl;
