library ieee;
use ieee.std_logic_1164.all;

library work;
use work.jddr_pkg.all;

entity jddr_parbiter is
generic
(
	NUMPORTS	: natural	:= 2
);
port
(
	-- To user ports
	clock		: in	std_logic_vector(NUMPORTS-1 downto 0);
	transrequest	: in	std_logic_vector(NUMPORTS-1 downto 0);
	transcomplete	: out	std_logic_vector(NUMPORTS-1 downto 0);
	ddr_address	: in	std_logic_vector(17*NUMPORTS-1 downto 0);
	ddr_start_block	: in	std_logic_vector(7*NUMPORTS-1 downto 0);
	ddr_end_block	: in	std_logic_vector(7*NUMPORTS-1 downto 0);
	readwriten	: in	std_logic_vector(NUMPORTS-1 downto 0);
	
	read_data	: out	std_logic_vector(143 downto 0);
	write_data	: in	std_logic_vector(144*NUMPORTS-1 downto 0);
	data_available	: out	std_logic_vector(NUMPORTS-1 downto 0);
	data_wanted	: out	std_logic_vector(NUMPORTS-1 downto 0);
	
	-- To DDR controller core
	r_transrequest	: out	std_logic;
	r_transcomplete	: in	std_logic;
	r_ddr_address	: out	std_logic_vector(16 downto 0);
	r_ddr_start_block	: out	std_logic_vector(6 downto 0);
	r_ddr_end_block	: out	std_logic_vector(6 downto 0);
	r_readwriten	: out	std_logic;

	r_read_data	: in	std_logic_vector(143 downto 0);
	r_write_data	: out	std_logic_vector(143 downto 0);
	r_data_available	: in	std_logic;
	r_data_wanted	: in	std_logic;
	r_data_clk	: in	std_logic
);
end jddr_parbiter;



architecture rtl of jddr_parbiter is
	signal cur_port	: integer range 0 to (NUMPORTS-1) := 0;
begin
	-- Data signal muxing (both directions at 133MHz)
	process (r_data_clk)
	begin
		if rising_edge(r_data_clk) then
			read_data <= r_read_data;

			for i in 0 to (NUMPORTS-1) loop
				if cur_port = i then
					data_available(i) <= r_data_available;
					data_wanted(i) <= r_data_wanted;
					r_write_data <= write_data(144*i+143 downto 144*i);
				else
					data_available(i) <= '0';
					data_wanted(i) <= '0';
				end if;
			end loop;
		end if;
	end process;

	-- Control signals from DDR to PortN domain
	process (clock(NUMPORTS-1 downto 0))
	begin
		for i in 0 to (NUMPORTS-1) loop
			if rising_edge(clock(i)) then
				if cur_port = i then
					transcomplete(i) <= r_transcomplete;
				else
					transcomplete(i) <= '0';
				end if;
			end if;
		end loop;
	end process;

	-- Control signals from PortN to DDR domain
	process (r_data_clk)
	begin
		if rising_edge(r_data_clk) then
			for i in 0 to (NUMPORTS-1) loop
				if cur_port = i then
					r_transrequest <= transrequest(i);
					r_ddr_address <= ddr_address(17*i+16 downto 17*i);
					r_ddr_start_block <= ddr_start_block(7*i+6 downto 7*i);
					r_ddr_end_block <= ddr_end_block(7*i+6 downto 7*i);
					r_readwriten <= readwriten(i);
					
					-- Advance current port
					if transrequest(i) = '0' and r_transcomplete = '0' then
						if NUMPORTS > 1 then
							if cur_port = (NUMPORTS-1) then
								cur_port <= 0;
							else
								cur_port <= cur_port + 1;
							end if;
						else
							cur_port <= 0;
						end if;
					end if;
				end if;
			end loop;
		end if;
	end process;
end rtl;
