LIBRARY ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

PACKAGE jddr_pkg IS
  component jddr
  generic
  (
      
      NUMPORTS : natural := 2;
      EXTRA_WRITE_LATENCY : natural := 0
  );
  port
  (
      -- Connect to a global clock (0 or 1) that isn't already
      -- connected to another PLL
      tm4_glbclk	: in std_logic;
      tm4_devbus	: inout std_logic_vector(40 downto 0);
      
      -- PortN request signals @ individual user clocks
      clock		: in	std_logic_vector(NUMPORTS-1 downto 0);
      transrequest	: in	std_logic_vector(NUMPORTS-1 downto 0);
      transcomplete	: out	std_logic_vector(NUMPORTS-1 downto 0);
      ddr_address	: in	std_logic_vector(17*NUMPORTS-1 downto 0);
      ddr_start_block	: in	std_logic_vector(7*NUMPORTS-1 downto 0);
      ddr_end_block	: in	std_logic_vector(7*NUMPORTS-1 downto 0);
      readwriten	: in	std_logic_vector(NUMPORTS-1 downto 0);
       
      -- PortN data signals @ 133mhz data_clk clock
      data_clk		: out	std_logic;
      read_data		: out	std_logic_vector(143 downto 0);
      write_data	: in	std_logic_vector(144*NUMPORTS-1 downto 0);
      data_available	: out	std_logic_vector(NUMPORTS-1 downto 0);
      data_wanted	: out	std_logic_vector(NUMPORTS-1 downto 0);
      
      -- DDR Module Interface Pin Signals
      tm4_ddr_DQ	: inout	std_logic_vector(71 downto 0);
      tm4_ddr_DQS	: inout	std_logic_vector(8 downto 0);
      tm4_ddr_DM	: out	std_logic_vector(8 downto 0);
      tm4_ddr_clk	: out	std_logic_vector(2 downto 0);
      tm4_ddr_A		: out	std_logic_vector(13 downto 0);
      tm4_ddr_RASn	: out	std_logic;
      tm4_ddr_CASn	: out	std_logic;
      tm4_ddr_WEn	: out	std_logic;
      tm4_ddr_Sn	: out	std_logic_vector(1 downto 0);
      tm4_ddr_BA	: out	std_logic_vector(1 downto 0);
      tm4_ddr_CLKE	: out	std_logic_vector(1 downto 0);
      tm4_ddr_RESETn	: out	std_logic;
      tm4_dqsclk_in	: in	std_logic;
      tm4_dqsclk_out	: out	std_logic
  );
  end component;

  -- jddr interface component declaration
  COMPONENT jddr_interface 
  GENERIC (
    EXTRA_WRITE_DELAY : NATURAL := 1);
  PORT (
    -- Globals Signals
    clk     : IN std_logic;
    reset_n : IN std_logic;

    -- DDR local side interface signals
    transrequest         : IN std_logic;
    readwriten           : IN std_logic;
    ddr_address          : IN STD_LOGIC_VECTOR(16 downto 0);
    transcomplete        : OUT STD_LOGIC;
    ddr_start_block      : IN STD_LOGIC_VECTOR(6 downto 0);
    ddr_end_block        : IN STD_LOGIC_VECTOR(6 downto 0);

    -- Local side data interface
    read_data    : OUT STD_LOGIC_VECTOR(143 downto 0);
    write_data   : IN STD_LOGIC_VECTOR(143 downto 0);
    data_available : OUT STD_LOGIC;
    data_wanted : OUT STD_LOGIC;
    data_clk     : OUT STD_LOGIC;

    -- DDR Module Interface Pin Signals
    tm4_ddr_DQ     : inout std_logic_vector(71 downto 0);
    tm4_ddr_DQS    : inout std_logic_vector(8 downto 0);
    tm4_ddr_DM     : out   std_logic_vector(8 downto 0);
    tm4_ddr_clk    : out   std_logic_vector(2 downto 0);
    tm4_ddr_A      : out std_logic_vector(13 downto 0);
    tm4_ddr_RASn   : out std_logic;
    tm4_ddr_CASn   : out std_logic;
    tm4_ddr_WEn    : out std_logic;
    tm4_ddr_Sn     : out std_logic_vector(1 downto 0);
    tm4_ddr_BA     : out std_logic_vector(1 downto 0);
    tm4_ddr_CLKE   : out std_logic_vector(1 downto 0);
    tm4_ddr_RESETn : out std_logic;
    tm4_dqsclk_in   : in std_logic;
    tm4_dqsclk_out   : out std_logic);
  END COMPONENT;

  -- Internal jddr controller component declarations
  COMPONENT jddr_pll PORT (
    inclk0 : IN STD_LOGIC;
    c0 : OUT STD_LOGIC;
    c1 : OUT STD_LOGIC;
    c2 : OUT STD_LOGIC;
    c3 : OUT STD_LOGIC;
    e0 : OUT STD_LOGIC;
    e1 : OUT STD_LOGIC;
    e2 : OUT STD_LOGIC;
    e3 : OUT STD_LOGIC );
  END COMPONENT;

  COMPONENT jddr_datapath PORT (
    -- globals
    clk            : IN std_logic;
    reset_n        : IN std_logic;
    dqs_outclk     : IN std_logic;
    write_clk      : IN std_logic;
    resynch_clk    : IN std_logic;
    postamble_clk  : IN std_logic;
    dqs_delay_ctrl : IN std_logic;

    -- DDR Controller Signals
    doing_wr    : IN  std_logic;
    wdata_low   : IN  std_logic_vector(71 DOWNTO 0);
    wdata_high  : IN  std_logic_vector(71 DOWNTO 0);
    doing_rd    : IN  std_logic;
    rdata_low   : OUT std_logic_vector(71 DOWNTO 0);
    rdata_high  : OUT std_logic_vector(71 DOWNTO 0);
    -- DDR Interface Signals
    ddr_dq   : INOUT std_logic_vector(71 DOWNTO 0);
    ddr_dqs  : INOUT std_logic_vector(8 DOWNTO 0);
    ddr_dm   : OUT   std_logic_vector(8 DOWNTO 0));
  END COMPONENT;

  COMPONENT  stratix_dll
  GENERIC (
    INPUT_FREQUENCY  : STRING;
    PHASE_SHIFT      : STRING := "72";
    SIM_INVALID_LOCK : NATURAL := 100000;
    SIM_VALID_LOCK   : NATURAL := 0;
    lpm_type         : STRING := "stratix_dll"
  );
  PORT (
    clk          : IN STD_LOGIC;
    delayctrlOUT : OUT STD_LOGIC
  );
  END COMPONENT;

  COMPONENT jddr_controller 
  GENERIC(EXTRA_WRITE_DELAY : natural := 1);
  PORT(
     -- SRAM buffer interface
     data_available : OUT STD_LOGIC;
     data_wanted : OUT STD_LOGIC;
     -- DDR Datapath interface
     doing_wr : OUT STD_LOGIC;
     doing_rd : OUT STD_LOGIC;
     -- DDR Signals
     ddr_clken : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
     ddr_csn : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
     ddr_rasn : OUT STD_LOGIC;
     ddr_casn : OUT STD_LOGIC;
     ddr_wen : OUT STD_LOGIC;
     ddr_address : OUT STD_LOGIC_VECTOR(13 DOWNTO 0);
     ddr_ba : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);

     -- Local interface signals
     transrequest, readwriten : IN STD_LOGIC;
     transcomplete : OUT STD_LOGIC;
     ddr_address_in : IN STD_LOGIC_VECTOR(16 DOWNTO 0);
     ddr_start_block : IN STD_LOGIC_VECTOR(6 DOWNTO 0);
     ddr_end_block : IN STD_LOGIC_VECTOR(6 DOWNTO 0);

     rstn : IN std_logic;
     clk : IN std_logic);
  END COMPONENT;

  COMPONENT jddr_counters 
    GENERIC(STARTUPDELAY : natural);
    PORT(
      -- Column Counter
      column_we : IN STD_LOGIC;
      column_start_block : IN STD_LOGIC_VECTOR(6 downto 0);
      column_end_block : IN STD_LOGIC_VECTOR(6 downto 0);
      column_write : OUT BOOLEAN;
      column_done  : OUT BOOLEAN;
      column_last  : OUT BOOLEAN;
      column       : OUT STD_LOGIC_VECTOR(9 downto 0);

      -- Startup / Refresh Counter
      clearstartup : IN STD_LOGIC;
      refreshcnten : IN STD_LOGIC;
      startup_delay : OUT BOOLEAN;
      nop200_delay : OUT BOOLEAN;

      -- Delay counter signals
      cleardelaycounter : IN STD_LOGIC;
      count_11 : OUT BOOLEAN;

      -- Refresh outstanding counter
      refreshA_outstanding : OUT BOOLEAN;
      refreshB_outstanding : OUT BOOLEAN;
      refreshA_now : OUT BOOLEAN;
      refreshB_now : OUT BOOLEAN;
      refreshedA : IN STD_LOGIC;
      refreshedB : IN STD_LOGIC;

      rstn : IN STD_LOGIC;
      clock : IN STD_LOGIC);
  END COMPONENT;

   
  component jddr_parbiter
  generic
  (
      NUMPORTS	: natural := 2
  );
  port
  (
      clock		: in	std_logic_vector(NUMPORTS-1 downto 0);
      transrequest	: in	std_logic_vector(NUMPORTS-1 downto 0);
      transcomplete	: out	std_logic_vector(NUMPORTS-1 downto 0);
      ddr_address	: in	std_logic_vector(17*NUMPORTS-1 downto 0);
      ddr_start_block	: in	std_logic_vector(7*NUMPORTS-1 downto 0);
      ddr_end_block	: in	std_logic_vector(7*NUMPORTS-1 downto 0);
      readwriten	: in	std_logic_vector(NUMPORTS-1 downto 0);
       
      read_data		: out	std_logic_vector(143 downto 0);
      write_data	: in	std_logic_vector(144*NUMPORTS-1 downto 0);
      data_available	: out	std_logic_vector(NUMPORTS-1 downto 0);
      data_wanted	: out	std_logic_vector(NUMPORTS-1 downto 0);
       
      r_transrequest	: out	std_logic;
      r_transcomplete	: in	std_logic;
      r_ddr_address	: out	std_logic_vector(16 downto 0);
      r_ddr_start_block	: out	std_logic_vector(6 downto 0);
      r_ddr_end_block	: out	std_logic_vector(6 downto 0);
      r_readwriten	: out	std_logic;

      r_read_data	: in	std_logic_vector(143 downto 0);
      r_write_data	: out	std_logic_vector(143 downto 0);
      r_data_available	: in	std_logic;
      r_data_wanted	: in	std_logic;
      r_data_clk	: in	std_logic
  );
  end component;

END PACKAGE jddr_pkg;
